Package: fsbrain
Type: Package
Title: Managing and Visualizing Brain Surface Data
Version: 0.6.0
Authors@R: person("Tim", "Schäfer", role = c("aut", "cre"), email = "ts+code@rcmd.org", comment = c(ORCID = "0000-0002-3683-8070"))
Maintainer: Tim Schäfer <ts+code@rcmd.org>
Description: Provides high-level access to neuroimaging data from standard software packages like 'FreeSurfer' <http://freesurfer.net/> on the level of subjects and groups. Load morphometry data, surfaces and brain parcellations based on atlases. Mask data using labels, load data for specific atlas regions only, and visualize data and statistical results directly in 'R'.
License: MIT + file LICENSE
Encoding: UTF-8
URL: https://github.com/dfsp-spirit/fsbrain
BugReports: https://github.com/dfsp-spirit/fsbrain/issues
Imports: reshape, freesurferformats (>= 0.1.17), pkgfilecache (>=
        0.1.1), rgl, squash, fields, viridis, data.table, magick,
        methods
Suggests: knitr, rmarkdown, testthat (>= 2.1.0), sphereplot (>= 1.5),
        misc3d, RColorBrewer, Rvcg (>= 0.20.2), igraph, pracma
VignetteBuilder: knitr
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-07-08 15:30:01 UTC; ts
Author: Tim Schäfer [aut, cre] (ORCID: <https://orcid.org/0000-0002-3683-8070>)
Repository: CRAN
Date/Publication: 2026-07-08 16:30:14 UTC
