% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_dti_tracks.R
\name{track.length}
\alias{track.length}
\title{Compute the total length of a path given by the coordinates of its points.}
\usage{
track.length(point_coords)
}
\arguments{
\item{point_coords}{n x 3 numerical matrix of 3D point coordinates, in the order of traversal.}
}
\value{
float, the sum of the length of all segments of the path.
}
\description{
Compute the total length of a path given by the coordinates of its points.
}
\keyword{internal}
