% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/brain_igraph.R
\name{fs.surface.as.adjacencylist}
\alias{fs.surface.as.adjacencylist}
\title{Turn surface mesh into a igraph and return its adjacency list representation.}
\usage{
fs.surface.as.adjacencylist(surface)
}
\arguments{
\item{surface}{an fs.surface instance as returned by \code{subject.surface}, an existing igraph (which will be returned as-is) or a string which is interpreted as a path to a surface file.}
}
\value{
list of integer vectors, the adjacency list.
}
\description{
Turn surface mesh into a igraph and return its adjacency list representation.
}
