% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spherical.R
\name{surf.center.fsaverage}
\alias{surf.center.fsaverage}
\title{Get pre-computed center for fsaverage white surface.}
\usage{
surf.center.fsaverage()
}
\value{
A numerical vector of length 3 with the x, y, and z coordinates of the center. The center was computed as the point halfway between the min and max mesh coordinates, on each axis separately.
}
\description{
Get pre-computed center for fsaverage white surface.
}
\note{
The coordinates are for the white surface and in surface space, i.e., based on the raw values stored in the \code{fsaverae/surf/lh.white} and \code{fsaverage/surf/rh.white} files, without applying any transformation.
}
\seealso{
\code{\link{surfs.props}}, which was used to compute the returned values.
}
\keyword{internal}
