% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_meshes.R
\name{get.rglstyle.glass}
\alias{get.rglstyle.glass}
\title{Get the glass visualization style parameters as a named list.}
\usage{
get.rglstyle.glass()
}
\value{
named list, style parameters that can be passed to \code{\link[rgl]{shade3d}} via \code{\link{do.call}}.
}
\description{
Glass-brain rendering style. This style has a very negative impact on rendering performance (especially in interactive mode). Hint: Run \code{\link[rgl]{material3d}} without arguments to see valid style keywords to create new styles.
}
\keyword{internal}
