% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spherical.R
\name{rgl.coord.lines}
\alias{rgl.coord.lines}
\title{Plot x, y and z axes in R,G,B.}
\usage{
rgl.coord.lines(len = 100)
}
\arguments{
\item{len}{numeric scalar or vector of length 3, length of axes. You can specify a negative value to see the negative directions.}
}
\value{
\code{NULL}, called for the plotting side effect.
}
\description{
Plot positive x, y, and z axes from the center to 'len'. Gets added to current plot. Useful for debugging and understanding the \code{rgl} coordinate system.
}
\note{
The x, y and z axes are plotted in red, green, and blue, respectively.
}
\keyword{internal}
