% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metadata_io.R
\name{get.subject.class}
\alias{get.subject.class}
\title{Construct FSGD Class name from group and non-continuous covariate columns.}
\usage{
get.subject.class(demographics_df, row_idx, class_columns, collapse = "_")
}
\arguments{
\item{demographics_df}{data.frame, as returned by \code{read.md.demographics} or created manually. Note that the data.frame must not contain any character columns, they should be converted to factors.}

\item{row_idx}{integer, the row in the df that belongs to this subject}

\item{class_columns}{the column names to use}

\item{collapse}{character string, the separator}
}
\value{
character string, the Class name for this subject, derived from the values in the 'class_columns'.
}
\description{
Construct FSGD Class name from group and non-continuous covariate columns.
}
\keyword{internal}
