% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{path.colors.from.orientation}
\alias{path.colors.from.orientation}
\title{Compute path color from its orientation.}
\usage{
path.colors.from.orientation(coords_list, use_three_colors_only = FALSE)
}
\arguments{
\item{coords_list}{list of \code{m} matrices, each \code{n} x 3 matrix must contain the 3D coords for one path.}

\item{use_three_colors_only}{logical, whether to use only three different colors, based on closest axis.}
}
\value{
\code{m} x 3 matrix, each row corresponds to a path and contains its color value (RGB, range 0..255).
}
\description{
Compute path color from its orientation.
}
\keyword{internal}
