% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/optdata.R
\name{download_optional_paper_data}
\alias{download_optional_paper_data}
\title{Download extra data to reproduce the figures from the fsbrain paper.}
\usage{
download_optional_paper_data(scheme = "https")
}
\arguments{
\item{scheme}{character string, the URL scheme to use. Either \code{"https"} (the default) or \code{"http"}. Switching to \code{"http"} can be useful as a fallback if the HTTPS server is unreachable.}
}
\value{
Named list. The list has entries: "available": vector of strings. The names of the files that are available in the local file cache. You can access them using get_optional_data_file(). "missing": vector of strings. The names of the files that this function was unable to retrieve.
}
\description{
Download extra data to reproduce the figures from the fsbrain paper.
}
\note{
Called for side effect of data download.
}
