% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hemilist.R
\name{hemilist.unwrap}
\alias{hemilist.unwrap}
\title{Unwrap hemi data from a named hemi list.}
\usage{
hemilist.unwrap(hemi_list, hemi = NULL, allow_null_list = FALSE)
}
\arguments{
\item{hemi_list}{named list, can have entries 'lh' and/or 'rh', see \code{\link{hemilist}}.}

\item{hemi}{character string, the hemi data name to retrieve from the list. Can be NULL if the list only has a single entry.}

\item{allow_null_list}{logical, whether to silently return NULL instead of raising an error if 'hemi_list' is NULL}
}
\value{
the data
}
\description{
Unwrap hemi data from a named hemi list.
}
\seealso{
Other hemilist functions: 
\code{\link{hemilist}()},
\code{\link{hemilist.derive.hemi}()},
\code{\link{hemilist.from.prefixed.list}()},
\code{\link{hemilist.get.combined.data}()},
\code{\link{hemilist.wrap}()},
\code{\link{is.hemilist}()}
}
\concept{hemilist functions}
