% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geodesic.R
\name{geodesic.average.distance}
\alias{geodesic.average.distance}
\title{Compute the average (pseudo-) geodesic distance on the mesh from each vertex to all other vertices.}
\usage{
geodesic.average.distance(surfaces)
}
\arguments{
\item{surfaces}{fs.surface instance or a \code{\link[fsbrain]{hemilist}} of the latter.}
}
\description{
Compute the average (pseudo-) geodesic distance on the mesh from each vertex to all other vertices.
}
\note{
This may take a while. It requires the 'Rvcg' package.
}
\keyword{internal}
