% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/brainview_magick.R
\name{image.remap.color}
\alias{image.remap.color}
\title{Remap a color in an image, typically used to set the background color to transparent.}
\usage{
\method{image}{remap.color}(
  source_img,
  source_color = NULL,
  source_point = "+1+1",
  target_color = "none"
)
}
\arguments{
\item{source_color}{the source color that should be replaced in the whole image. Set to NULL to disable.}

\item{source_point}{the source pixel in which to start the flood filling. Set to NULL to disable.}

\item{target_color}{an image magick color string, use 'none' for transparency. Only used with flood fill.}
}
\description{
Offers 2 algorithm: remap color by flood-filling from a given pixel, or remap a hardcoded color throughout the entire image. Provide one of 'source_color' or 'source_point' by setting the other to NULL. If both are given, source_color takes precedence and source_point is silently ignored.
}
\keyword{internal}
