% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hemilist.R
\name{hemilist.derive.hemi}
\alias{hemilist.derive.hemi}
\title{Derive 'hemi' string from the data in a hemilist}
\usage{
hemilist.derive.hemi(hemilist)
}
\arguments{
\item{hemilist}{hemilist, an existing hemilist}
}
\value{
character string, one of 'lh', 'rh' or 'both'
}
\description{
Derive 'hemi' string from the data in a hemilist
}
\note{
See \code{\link{hemilist}} for details.
}
\seealso{
Other hemilist functions: 
\code{\link{hemilist}()},
\code{\link{hemilist.from.prefixed.list}()},
\code{\link{hemilist.get.combined.data}()},
\code{\link{hemilist.unwrap}()},
\code{\link{hemilist.wrap}()},
\code{\link{is.hemilist}()}
}
\concept{hemilist functions}
