% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rglactions.R
\name{shift.hemis.rglactions}
\alias{shift.hemis.rglactions}
\title{Shift hemis apart if indicated in rglactions}
\usage{
shift.hemis.rglactions(coloredmeshes, rglactions)
}
\arguments{
\item{coloredmeshes}{hemilist of coloredmeshes}

\item{rglactions}{the \code{\link{rglactions}}, a named list as passed to functions like vis.subject.morph.native.}
}
\value{
hemilist of coloredmeshes, the coordinates may or may not have been shifted, depending on the \code{\link{rglactions}}.
}
\description{
Shift hemis apart if indicated in rglactions
}
\keyword{internal}
