% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cbar.R
\name{can.plot.colorbar.from.coloredmeshes}
\alias{can.plot.colorbar.from.coloredmeshes}
\title{Determine whether colorbar can be plotted with given coloredmeshes.}
\usage{
can.plot.colorbar.from.coloredmeshes(coloredmeshes)
}
\arguments{
\item{coloredmeshes}{hemilist of coloredmeshes}
}
\value{
logical, whether the metadata suffices to plot a colorbar
}
\description{
Determine whether colorbar can be plotted with given coloredmeshes.
}
\keyword{internal}
