% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coloredmesh.R
\name{fs.coloredmesh}
\alias{fs.coloredmesh}
\title{fs.coloredmesh constructor}
\usage{
fs.coloredmesh(
  mesh,
  col,
  hemi,
  render = TRUE,
  metadata = NULL,
  add_normals = FALSE
)
}
\arguments{
\item{mesh}{a \code{mesh3d} instance as returned by \code{\link[rgl]{tmesh3d}} or an \code{fs.surface} brain surface mesh as returned by functions like \code{\link[fsbrain]{subject.surface}}.}

\item{col}{vector of vertex colors for the mesh, one color per vertex. Expanded if exactly one color.}

\item{hemi}{character string, one of 'lh' or 'rh'. This may be used by visualization functions to decide whether or not to show this mesh in a certain view.}

\item{render}{logical, whether to render this mesh during visualization}

\item{metadata}{optional, named list containing metadata}

\item{add_normals}{logical, whether to compute normals and save them in the mesh.}
}
\value{
an \code{fs.coloredmesh} instance. The only fields one should use in client code are 'mesh', 'hemi' and 'col', all others are considered internal and may change without notice.
}
\description{
fs.coloredmesh constructor
}
