% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_group.R
\name{recycle}
\alias{recycle}
\title{Recycle parameters or whatever.}
\usage{
recycle(x, times)
}
\arguments{
\item{x}{a vector of whatever}

\item{times}{positive integer, the required length of the output}
}
\value{
a vector of length 'times', with the recycled 'x'
}
\description{
Recycle parameters or whatever.
}
\note{
Todo: most likely there is an R function for this already, find it.
}
\keyword{internal}
