% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metadata_io.R
\name{report.on.demographics}
\alias{report.on.demographics}
\title{Print a demographics report}
\usage{
report.on.demographics(
  demographics_df,
  group_column_name = NULL,
  paired = FALSE
)
}
\arguments{
\item{demographics_df}{a demographics data.frame, as returned by  \code{\link[fsbrain]{read.md.demographics}}.}

\item{group_column_name, }{string or NULL. If given, the column name of the group column. It must be a factor column with 2 levels. Enables group-comparison tests. Defaults to \code{NULL}.}

\item{paired}{Whether the data of the two groups if paired (repeated measurements). Only relevant if group_column_name is given and tests for group differences are included in the report. Defaults to \code{FALSE}.}
}
\value{
vector of character strings, the lines of the demographics report.
}
\description{
Print a demographics report
}
\seealso{
Other metadata functions: 
\code{\link{demographics.to.fsgd.file}()},
\code{\link{read.md.demographics}()},
\code{\link{read.md.subjects}()}
}
\concept{metadata functions}
