% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/curvature.R
\name{shape.descriptor.names}
\alias{shape.descriptor.names}
\title{Get all shape descriptor names.}
\usage{
shape.descriptor.names()
}
\value{
vector of character strings, the names
}
\description{
Get all shape descriptor names.
}
