% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{rad2deg}
\alias{rad2deg}
\title{Convert raduians to degree}
\usage{
rad2deg(rad)
}
\description{
Convert raduians to degree
}
\keyword{internal}
