% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/morph_atlas_agg.R
\name{fs.value.list.from.agg.res}
\alias{fs.value.list.from.agg.res}
\title{Create a named value list from a dataframe.}
\usage{
fs.value.list.from.agg.res(agg_res, subject_id)
}
\arguments{
\item{agg_res, }{a dataframe. The result of calling group.agg.atlas.native().}

\item{subject_id, }{string. A subject identifier, must occur in the subject column of the dataframe agg_res.}
}
\value{
region_value_list, named list of strings. Each name must is a region name from the annotation, and the value is a scalar that resulting from aggregating the morphometry data for that region and subject.
}
\description{
Given the result of the group.agg.atlas.native() function, extract a named region value list (typically for use with the spread.values.over.annot() function) for a single subject.
}
\keyword{internal}
