% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis.R
\name{per.hemi.vertex.indices}
\alias{per.hemi.vertex.indices}
\title{Transform surfaces indices which go over two surfaces to per-hemi indices.}
\usage{
per.hemi.vertex.indices(surfaces, vertices)
}
\arguments{
\item{surfaces}{hemilist of surfaces, or a single integer, which will be interpreted as the number of vertices of the left hemisphere surface.}

\item{vertices}{positive integer vector, the query vertex indices. Must be in range 1 to (num_verts_lh + num_verts_rh).}
}
\value{
named list with the following entries: 'vertices', hemilist of indices for the hemispheres. 'query_indices', hemilist of the indices of the respective vertices in the vector that was passed as parameter vertices. 'vertices_hemi': vector of character strings containing the hemi value (lh or rh) for the query vertices.
}
\description{
Transform surfaces indices which go over two surfaces to per-hemi indices.
}
\keyword{internal}
