% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_volume_3d.R
\name{is.fs.coloredvoxels}
\alias{is.fs.coloredvoxels}
\title{Check whether object is an fs.coloredvoxels instance (S3)}
\usage{
is.fs.coloredvoxels(x)
}
\arguments{
\item{x}{any \code{R} object}
}
\value{
TRUE if its argument is a fs.coloredvoxels instance (that is, has "fs.coloredvoxels" among its classes) and FALSE otherwise.
}
\description{
Check whether object is an fs.coloredvoxels instance (S3)
}
