% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cbar.R
\name{mkco.div}
\alias{mkco.div}
\title{Return recommended 'makecmap_options' for diverging data.}
\usage{
mkco.div()
}
\value{
named list, visualization settings to be used as 'makecmap_options' for diverging data.
}
\description{
This function returns recommended visualization settings (a colormap function and suitable other settings) for the given type of data. The return value is meant to be passed as parameter 'makecmap_options' to the vis.* functions, e.g., \code{\link[fsbrain]{vis.subject.morph.native}}.
}
