% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{mesh.vertex.included.faces}
\alias{mesh.vertex.included.faces}
\title{Return all faces which are made up completely of the listed vertices.}
\usage{
mesh.vertex.included.faces(surface_mesh, source_vertices)
}
\arguments{
\item{surface_mesh}{surface mesh, as loaded by \code{\link[fsbrain]{subject.surface}} or \code{\link[freesurferformats]{read.fs.surface}}.}

\item{source_vertices}{integer vector, the vertex indices.}
}
\value{
integer vector, the face indices
}
\description{
Return all faces which are made up completely of the listed vertices.
}
\seealso{
Other surface mesh functions: 
\code{\link{face.edges}()},
\code{\link{label.border}()},
\code{\link{mesh.vertex.neighbors}()},
\code{\link{subject.surface}()},
\code{\link{vis.path.along.verts}()}
}
\concept{surface mesh functions}
\keyword{internal}
