% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/curvature.R
\name{surface.curvatures}
\alias{surface.curvatures}
\title{Compute the k1 and k2 principal curvatures of a mesh.}
\usage{
surface.curvatures(surface)
}
\arguments{
\item{surface}{an fs.surface instance, as returned by \code{subject.surface}.}
}
\value{
named list, the entries 'K1' and 'K2' contain the principal curvatures.
}
\description{
Compute the k1 and k2 principal curvatures of a mesh.
}
\note{
Require the optional dependency package 'Rvcg'.
}
