% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_volume.R
\name{boxcoords.from.bbox}
\alias{boxcoords.from.bbox}
\title{Compute the coordinates of the 8 corners of a 3D box.}
\usage{
boxcoords.from.bbox(axes_min, axes_max)
}
\arguments{
\item{axes_min}{numerical vector of length 3, the min values of the 3 axes}

\item{axes_max}{numerical vector of length 3, the max values of the 3 axes}
}
\value{
numerical matrix with 3 columns and 8 rows, the edge coordinates
}
\description{
Given the extreme values (min and max) along the 3 axes, compute the coordinates of the 8 corners of a 3D box.
}
\keyword{internal}
