% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis.R
\name{hemlist.ensure.contains}
\alias{hemlist.ensure.contains}
\title{title Ensure an key for a hemilist exists.}
\usage{
hemlist.ensure.contains(hemilist, required_hemi, error_tag = "")
}
\arguments{
\item{hemilist}{a hemilist}

\item{required_hemi}{string, one of 'lh', 'rh' or 'both'.}
}
\description{
title Ensure an key for a hemilist exists.
}
\keyword{internal}
