% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_volume.R
\name{extract.volume.3D}
\alias{extract.volume.3D}
\title{Try to extract a 3D volume from the input argument.}
\usage{
extract.volume.3D(stats, silent = getOption("fsbrain.silent", default = FALSE))
}
\arguments{
\item{stats}{a 3D array, 4D array or a string that can be treated as a filename to a volume image containing such an array.}

\item{silent}{logical, whether to suppress file reading messages.}
}
\value{
the obtained 3D array
}
\description{
Check whether it already is such an array, whether it is a filename that can be loaded with \code{freesurferformats::read.fs.volume} into such an array, etc.
}
\note{
This function stops with an error if the input cannot be returned as a 3D array.
}
\keyword{internal}
