% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rglactions.R
\name{rglactions.transform}
\alias{rglactions.transform}
\title{Apply data transformation rglactions.}
\usage{
rglactions.transform(measure_data, rglactions)
}
\arguments{
\item{measure_data}{numeric vector, or hemilist of numeric vectors. The data, usually vertex-wise morphometry data.}

\item{rglactions}{named list, passed as parameter '\code{\link{rglactions}}' to functions like \code{\link[fsbrain]{vis.subject.morph.native}}.}
}
\value{
the transformed data
}
\description{
Apply data transformation rglactions.
}
\keyword{internal}
