% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coloredmesh.R
\name{fs.surface.to.tmesh3d}
\alias{fs.surface.to.tmesh3d}
\title{Get an rgl tmesh3d instance from a brain surface mesh.}
\usage{
fs.surface.to.tmesh3d(surface)
}
\arguments{
\item{surface}{an fs.surface instance, as returned by \code{subject.surface} or \code{freesurferformats::read.fs.surface}.}
}
\value{
a tmesh3d instance, see \code{rgl::tmesh3d} for details.
}
\description{
Get an rgl tmesh3d instance from a brain surface mesh.
}
