% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_meshes.R
\name{fsbrain.renderable}
\alias{fsbrain.renderable}
\title{Check whether object can be rendered by fsbrain}
\usage{
fsbrain.renderable(x)
}
\arguments{
\item{x}{any \code{R} object}
}
\value{
TRUE if \emph{x} is an instance of a class that can be rendered by fsbrain visualization functions, and FALSE otherwise. Currently, the following types are renderable: \code{fs.coloredvoxels}, \code{fs.coloredmesh}, \code{Triangles3D}.
}
\description{
Check whether object can be rendered by fsbrain
}
\seealso{
\code{\link[fsbrain]{is.Triangles3D}}
}
\keyword{internal}
