% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{flc}
\alias{flc}
\title{Given a list of path coordinates, create matrix containing only the first and last point of each path.}
\usage{
flc(coords_list)
}
\arguments{
\item{coords_list}{list of \code{m} matrices, each \code{n} x 3 matrix must contain the 3D coords for one path.}
}
\value{
m x 6 numeric matrix, containing the first and last point of a path per row (two 3D xyz-coords, so 6 values per row).
}
\description{
Given a list of path coordinates, create matrix containing only the first and last point of each path.
}
\keyword{internal}
