% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{rglo}
\alias{rglo}
\title{Get rgloptions and consider global options.}
\usage{
rglo()
}
\value{
named list, usable as 'rgloptions' parameter for vis functions like \code{\link[fsbrain]{vis.subject.morph.native}}.
}
\description{
This function retrieves the global rgloptions defined in \code{getOption('fsbrain.rgloptions')}, or, if this is not set, returns the value from \code{\link{rglot}}.
}
\note{
You can set the default size for all fsbrain figures to 1200x1200 pixels like this: \code{options("fsbrain.rgloptions"=list("windowRect"=c(50,50,1200,1200)))}.
}
