% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_volume.R
\name{vol.planes}
\alias{vol.planes}
\title{Translate names and indices of planes.}
\usage{
vol.planes(plane = NULL)
}
\arguments{
\item{plane}{NULL, a plane index, or a plane name.}
}
\value{
if \code{plane} is NULL, all available planes and their indices as a named list. If \code{plane} is an integer (a plane index), its name. If \code{plane} is an characters string (a plane name), its index.
}
\description{
Translate names and indices of 3D image planes. The names only make sense if the data in the volume is in the default FreeSurfer conformed orientation.
}
\seealso{
Other volume utility: 
\code{\link{vol.boundary.box}()},
\code{\link{vol.imagestack}()},
\code{\link{vol.merge}()},
\code{\link{vol.overlay.colors.from.activation}()},
\code{\link{vol.slice}()}
}
\concept{volume utility}
