% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/smooth.R
\name{surf.sphere.spatialfilter}
\alias{surf.sphere.spatialfilter}
\title{Apply spatial filter to surface data.}
\usage{
surf.sphere.spatialfilter(source_data, sphere_dists, gaussian_weights)
}
\arguments{
\item{source_data}{numerical vector, per-vertex data for a surface.}

\item{sphere_dists}{named list with 3 entries, as returned by \code{surf.sphere.dist}}

\item{gaussian_weights}{list of double vectors, the Gaussian weights for all neighbors of the respective vertex. As returned by \code{surf.sphere.gaussianweights}.}
}
\value{
numerical vector, the spatially filtered per-vertex data.
}
\description{
Apply spatial filter to surface data.
}
\keyword{internal}
