% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_meshes.R
\name{get.rglstyle.shiny}
\alias{get.rglstyle.shiny}
\title{Get a shiny visualization style.}
\usage{
get.rglstyle.shiny()
}
\value{
named list, style parameters that can be passed to to \code{\link[rgl]{shade3d}} via \code{\link{do.call}}.
}
\description{
A shiny or glossy rendering style. Looks a bit more modern, but the resulting highlights may make the interpretation of the plotted data a bit harder in some areas. Hint: Run \code{\link[rgl]{material3d}} without arguments to see valid style keywords to create new styles.
}
\keyword{internal}
