% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/optdata.R
\name{sjd.demo}
\alias{sjd.demo}
\title{Download optional demo data if needed and return its path.}
\usage{
sjd.demo(accept_freesurfer_license = FALSE)
}
\arguments{
\item{accept_freesurfer_license}{logical, whether you want to also download fsaverage and fsaverage3, and accept the FreeSurfer license for fsaverage and fsaverage3, available at https://surfer.nmr.mgh.harvard.edu/fswiki/FreeSurferSoftwareLicense. Defaults to FALSE. If FALSE, only the demo data from fsbrain itself ('subject1') will be downloaded.}
}
\value{
character string, the path to the 'subjects_dir' directory within the downloaded optional data directory.
}
\description{
This is a wrapper around \code{download_optional_data()} and \code{get_optional_data_filepath("subjects_dir")}. It will download the optional fsbrain demo data unless it already exists locally.
}
\note{
This function will stop if the data cannot be accessed, i.e., the 'subjects_dir' does not exist after trying to download the data.
}
