% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spherical.R
\name{eeg_coords}
\alias{eeg_coords}
\title{Internal function to get some demo EEG electrode coordinates. Will be removed from public API. Do not use this.}
\usage{
eeg_coords(label_subset = NULL)
}
\arguments{
\item{label_subset}{vector of character strings, electrode names like 'Nz' or 'RPA'. ÖLeave at the default value \code{NULL} to get all available ones.}
}
\value{
data.frame with one row per electrode, and the following 3 columns: 'label': the electrode name, 'theta': the azimuth in degrees, 'phi': the latitude in degrees.
}
\description{
Internal function to get some demo EEG electrode coordinates. Will be removed from public API. Do not use this.
}
\note{
There are lots of different naming conventions for spherical coordinates, see https://en.wikipedia.org/wiki/Spherical_coordinate_system.
}
\references{
See http://wiki.besa.de/index.php?title=Electrodes_and_Surface_Locations
}
\keyword{internal}
