% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_volume.R
\name{vol.boundary.box.apply}
\alias{vol.boundary.box.apply}
\title{Apply a boundary box to a volume, returning the inner volume part}
\usage{
vol.boundary.box.apply(volume, bbox)
}
\arguments{
\item{volume}{a 3D image volume}

\item{bbox}{the boundary box}
}
\value{
a 3D image volume, the inner volume part, resulting from the application of the boundary box
}
\description{
Apply a boundary box to a volume, returning the inner volume part
}
