% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cbar.R
\name{coloredmeshes.combined.data.range}
\alias{coloredmeshes.combined.data.range}
\title{Retrieve combined data range from hemilist of coloredmeshes.}
\usage{
coloredmeshes.combined.data.range(coloredmeshes)
}
\arguments{
\item{coloredmeshes}{hemilist of coloredmeshes}
}
\value{
numeric vector of length 2, the finite data range
}
\description{
Retrieve combined data range from hemilist of coloredmeshes.
}
\keyword{internal}
