% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_meshes.R
\name{sortcoloredmeshes.by.hemi}
\alias{sortcoloredmeshes.by.hemi}
\title{Sort coloredmeshes into 2 lists by their 'hemi' property.}
\usage{
sortcoloredmeshes.by.hemi(coloredmeshes)
}
\arguments{
\item{coloredmeshes}{list of coloredmeshes or other renderables}
}
\value{
named list with two entries: "lh": list of coloredmeshes that have property hemi set to 'lh'. "rh": list of coloredmeshes that have property hemi set to 'rh'. The rest is ignored.
}
\description{
Sort coloredmeshes into 2 lists by their 'hemi' property.
}
\keyword{internal}
