% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/seg_stats.R
\name{qc.from.segstats.table}
\alias{qc.from.segstats.table}
\title{Perform data quality check based on a segstats table.}
\usage{
qc.from.segstats.table(filepath, ...)
}
\arguments{
\item{filepath}{path to input file, a tab-separated file generated by running the FreeSurfer tools 'aparcstats2table' or 'asegstats2table'. The command line in the system shell would be something like 'aparcstats2table_bin --subjectsfile $subjects_file --meas $measure --hemi $hemi -t $aparc_output_table'.}

\item{...}{parameters passed to \code{\link[fsbrain]{qc.from.regionwise.df}}.}
}
\value{
see \code{\link[fsbrain]{qc.from.regionwise.df}}
}
\description{
Perform data quality check based on a segstats table.
}
\seealso{
Other quality check functions: 
\code{\link{qc.for.group}()},
\code{\link{qc.from.regionwise.df}()}
}
\concept{quality check functions}
\keyword{internal}
