% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_volume_3d.R
\name{print.fs.coloredvoxels}
\alias{print.fs.coloredvoxels}
\title{Print description of fs.coloredvoxels (S3).}
\usage{
\method{print}{fs.coloredvoxels}(x, ...)
}
\arguments{
\item{x}{brain voxel tris with class \code{fs.coloredvoxels}.}

\item{...}{further arguments passed to or from other methods}
}
\description{
Print description of fs.coloredvoxels (S3).
}
