% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cbar.R
\name{cm.div}
\alias{cm.div}
\title{Return the standard fsbrain diverging colormap.}
\usage{
cm.div(report = FALSE)
}
\arguments{
\item{report}{logical, whether to print a message with a name of the chosen colormap, in format \code{package::function#palette}.}
}
\description{
Return the standard fsbrain diverging colormap.
}
\note{
Returns some diverging palette, suitable for visualizing data that is centered around zero.
}
