% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metadata_io.R
\name{fslong.subjects.detect}
\alias{fslong.subjects.detect}
\title{Get subject names from sub directories of FreeSurfer long directory.}
\usage{
fslong.subjects.detect(subjects_dir, timepoint_names = c("_MR1", "_MR2"))
}
\arguments{
\item{subjects_dir}{character string, path to a single recon-all longitudinal output dir from FreeSurfer.}
}
\description{
Find all subject names for which the FreeSurfer longitudinal pipeline may have finished. These are the subjects that have the \code{_MR1} and \code{_MR2} directories.
}
\keyword{internal}
