% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{fup}
\alias{fup}
\title{Transform first character of a string to uppercase.}
\usage{
fup(word)
}
\arguments{
\item{word, }{string. Any string.}
}
\value{
string, the input string with the first character transformed to uppercase.
}
\description{
Transform first character of a string to uppercase. This is useful when labeling plots. Important: this function does not know about different encodings, languages or anything, it just calls \code{\link{toupper}} for the first character.
}
\examples{
   word_up = fup("word");

}
