% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers_magick.R
\name{images.dimmax}
\alias{images.dimmax}
\title{Compute max width and height of magick images.}
\usage{
images.dimmax(images)
}
\arguments{
\item{images}{a vector/stack of magick images. See \code{magick::image_blank} or other methods to get one.}
}
\value{
named list with entries 'width' and 'height'
}
\description{
Compute max width and height of magick images.
}
\keyword{internals}
