% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_meshes.R
\name{vis.renderable}
\alias{vis.renderable}
\title{Visualize a renderable object}
\usage{
vis.renderable(cmesh, skip_all_na = TRUE, style = "default")
}
\arguments{
\item{cmesh}{an instance of one of the supported renderable classes}

\item{skip_all_na}{logical, whether to skip rendering hidden instances}

\item{style}{a rendering style, can be a style name or a list defining an rgl material style}
}
\description{
Renders instances of \code{coloredmesh}, \code{coloredvoxels} and \code{Triangles3D}.
}
\seealso{
\code{\link[fsbrain]{fsbrain.renderable}}
}
\keyword{internal}
