% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{colorlist.brain.clusters}
\alias{colorlist.brain.clusters}
\title{Return diverging color list}
\usage{
colorlist.brain.clusters(num_colors)
}
\arguments{
\item{num_colors}{integer, the number of colors you want}
}
\value{
vector of colors
}
\description{
Return diverging color list
}
