% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hemilist.R
\name{is.hemilist}
\alias{is.hemilist}
\title{Check whether x is a hemilist}
\usage{
is.hemilist(x)
}
\arguments{
\item{x}{any R object}
}
\value{
whether 'x' is a hemilist
}
\description{
A hemilist is a named list with entries 'lh' and/or 'rh', see \code{\link{hemilist}}.
}
\seealso{
Other hemilist functions: 
\code{\link{hemilist}()},
\code{\link{hemilist.derive.hemi}()},
\code{\link{hemilist.from.prefixed.list}()},
\code{\link{hemilist.get.combined.data}()},
\code{\link{hemilist.unwrap}()},
\code{\link{hemilist.wrap}()}
}
\concept{hemilist functions}
