% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_surface_background.R
\name{collayer.from.annotdata}
\alias{collayer.from.annotdata}
\title{Compute surface color layer from annotation or atlas data.}
\usage{
collayer.from.annotdata(lh_annotdata = NULL, rh_annotdata = NULL)
}
\arguments{
\item{lh_annotdata}{loaded annotation data for left hemi, as returned by \code{\link[fsbrain]{subject.annot}}}

\item{rh_annotdata}{loaded annotation data for right hemi}
}
\value{
named hemi list, each entry is a vector of color strings, one color per surface vertex. The coloring represents the atlas data.
}
\description{
Compute surface color layer from annotation or atlas data.
}
\seealso{
You can plot the return value using \code{\link[fsbrain]{vis.color.on.subject}}.

Other surface color layer: 
\code{\link{collayer.bg}()},
\code{\link{collayer.bg.atlas}()},
\code{\link{collayer.bg.meancurv}()},
\code{\link{collayer.bg.sulc}()},
\code{\link{collayer.from.annot}()},
\code{\link{collayer.from.mask.data}()},
\code{\link{collayer.from.morphlike.data}()},
\code{\link{collayers.merge}()}
}
\concept{surface color layer}
