% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_volume.R
\name{normalize}
\alias{normalize}
\title{Normalize data.}
\usage{
normalize(x)
}
\arguments{
\item{x}{the data}
}
\value{
the scaled data
}
\description{
Scales data to the range \verb{[0, 1]} based on min and max values.
}
\keyword{internal}
