% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cbar.R
\name{cm.seq}
\alias{cm.seq}
\title{Return the standard fsbrain sequential colormap.}
\usage{
cm.seq(report = FALSE)
}
\arguments{
\item{report}{logical, whether to print a message with a name of the chosen colormap, in format \code{package::function#palette}.}
}
\description{
Return the standard fsbrain sequential colormap.
}
\note{
This returns a sequential, multi-hue palette.
}
