% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cbar.R
\name{coloredmeshes.get.md}
\alias{coloredmeshes.get.md}
\title{Retrieve metadata from hemilist of coloredmeshes.}
\usage{
coloredmeshes.get.md(coloredmeshes, mdname)
}
\arguments{
\item{coloredmeshes}{hemilist of coloredmeshes}

\item{mdname}{the key in the named metadata list}
}
\value{
the metadata value at the given key/mdname
}
\description{
Retrieve metadata from hemilist of coloredmeshes.
}
\keyword{internal}
