% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/morph_atlas_agg.R
\name{pp.named.list}
\alias{pp.named.list}
\title{Pretty-print a named list or vector.}
\usage{
pp.named.list(named_list)
}
\arguments{
\item{named_list}{a named list or vector}
}
\value{
character string, the printed list
}
\description{
Pretty-print a named list or vector.
}
\keyword{internal}
