% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metadata_io.R
\name{deepcopylist.long}
\alias{deepcopylist.long}
\title{Write deepcopy list for longitudinal subjects.}
\usage{
deepcopylist.long(
  measures = c("thickness", "area", "volume"),
  fwhms = c("5", "10", "15"),
  hemis = c("lh", "rh"),
  long_measures = c("avg", "rate", "spc", "pc1"),
  template = "fsaverage",
  has_stacked_file = TRUE,
  output_file = NULL
)
}
\value{
vector of character strings, the file entries. Set ouput_file to also write them to a file.
}
\description{
Write deepcopy list for longitudinal subjects.
}
\keyword{internal}
