% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/optdata.R
\name{download_fsaverage_minimal}
\alias{download_fsaverage_minimal}
\title{Download only essential FreeSurfer v6 fsaverage files for quick visualization.}
\usage{
download_fsaverage_minimal(accept_freesurfer_license = FALSE, scheme = "https")
}
\arguments{
\item{accept_freesurfer_license}{logical, whether you accept the FreeSurfer license for fsaverage, available at https://surfer.nmr.mgh.harvard.edu/fswiki/FreeSurferSoftwareLicense. Defaults to FALSE.}

\item{scheme}{character string, the URL scheme to use. Either \code{"https"} (the default) or \code{"http"}. Switching to \code{"http"} can be useful as a fallback if the HTTPS server is unreachable.}
}
\value{
Named list. The list has entries: "available": vector of strings. The names of the files that are available in the local file cache. You can access them using get_optional_data_file(). "missing": vector of strings. The names of the files that this function was unable to retrieve.
}
\description{
Download a minimal subset of files from the FreeSurfer v6 fsaverage subject, containing only the white surfaces, cortex labels, and curvature files needed for basic 3D visualization (e.g., in a Shiny app). This is much faster than \code{\link{download_fsaverage}} which downloads 19 files. The files are subject to the FreeSurfer software license, see parameter 'accept_freesurfer_license' for details.
}
\note{
This downloads approximately 8 files instead of the 19 downloaded by \code{download_fsaverage}, making it suitable for environments with startup time constraints (e.g., Posit Connect Cloud, shinyapps.io).
}
