% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/optdata.R
\name{delete_all_optional_data}
\alias{delete_all_optional_data}
\title{Delete all data in the package cache.}
\usage{
delete_all_optional_data()
}
\value{
integer. The return value of the unlink() call: 0 for success, 1 for failure. See the unlink() documentation for details.
}
\description{
Delete all data in the package cache.
}
