% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{rglot}
\alias{rglot}
\title{Get rgloptions for testing.}
\usage{
rglot()
}
\value{
named list, usable as 'rgloptions' parameter for vis functions like \code{\link[fsbrain]{vis.subject.morph.native}}.
}
\description{
This function defines the figure size that is used during the unit tests. Currently \code{list('windowRect' = c(50, 50, 800, 800)}.
}
