% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_meshes.R
\name{get.rglstyle}
\alias{get.rglstyle}
\title{Get the default visualization style parameters as a named list.}
\usage{
get.rglstyle(style)
}
\arguments{
\item{style}{string. A style name. Available styles are one of: "default", "shiny", "semitransparent", "glass", "edges".}
}
\value{
a style, resolved to a parameter list compatible with \code{\link[rgl]{material3d}}.
}
\description{
Run \code{\link[rgl]{material3d}} without arguments to see valid style keywords to create new styles.
}
\seealso{
\code{\link[rgl]{shade3d}} can use the returned style
}
