% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metadata_io.R
\name{qdec.table.filter}
\alias{qdec.table.filter}
\title{Filter QDEC long table for subjects.}
\usage{
qdec.table.filter(qdec_file, subjects_list, output_qdec_file = NULL)
}
\arguments{
\item{qdec_file}{the source QDEC table}

\item{subjects_list}{the subjects to extract from the QDEC file}

\item{output_qdec_file}{optional character string, a file name to which to write the resulting, filtered table. If not given, no file is created.}
}
\value{
the data.frame containing the subset of subjects from the original QDEC file.
}
\description{
Filter QDEC long table for subjects.
}
\note{
This assumes that there are 2 time points per subject and warns if not all requested subjects were found.
}
\keyword{internal}
