% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_meshes.R
\name{vis.coloredmesh}
\alias{vis.coloredmesh}
\title{Draw a coloredmesh using a style.}
\usage{
vis.coloredmesh(cmesh, style = "default")
}
\arguments{
\item{cmesh}{a coloredmesh. A coloredmesh is a named list as returned by the coloredmesh.from.* functions. It has the entries 'mesh' of type tmesh3d, a 'col', which is a color specification for such a mesh.}

\item{style}{a named list of style parameters or a string specifying an available style by name (e.g., 'shiny'). Defaults to 'default', the default style. Pass the magic word 'from_mesh' to try to retrieve a style (as a name or a style list) from the field \code{style} of the mesh, or default to "default" if the mesh has no such field.}
}
\description{
Draw a coloredmesh using a style.
}
\seealso{
\code{\link[fsbrain]{vis.renderable}}
}
\keyword{internal}
