% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_surface_background.R
\name{common.makecmap.range}
\alias{common.makecmap.range}
\title{Get cmap and colorlayer from data and makecmap_options.}
\usage{
common.makecmap.range(
  makecmap_options,
  lh_data = NULL,
  rh_data = NULL,
  return_metadata = FALSE
)
}
\arguments{
\item{makecmap_options}{named list of parameters to pass to \code{\link[squash]{makecmap}}. Must not include the unnamed first parameter, which is derived from 'measure'.}

\item{lh_data}{numeric vector, data for left hemisphere.}

\item{rh_data}{numeric vector, data for right hemisphere.}

\item{return_metadata}{logical, whether to return additional metadata as entry 'metadata' in the returned list}
}
\value{
named list, with entries 'map': named list, the squash cmap, and 'collayer': hemilist of color vectors
}
\description{
Applies a requested 'range' setting if present in makecmap_options. A shared colormap is used for the data of both hemispheres (if present).
}
\keyword{internal}
