% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coloredmesh.R
\name{print.fsbrain}
\alias{print.fsbrain}
\title{Print description of an fsbrain (S3).}
\usage{
\method{print}{fsbrain}(x, ...)
}
\arguments{
\item{x}{fsbrain instance with class \code{fsbrain}.}

\item{...}{further arguments passed to or from other methods}
}
\description{
Print description of an fsbrain (S3).
}
