% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metadata_io.R
\name{read.md.subjects.from.fsgd}
\alias{read.md.subjects.from.fsgd}
\title{Read subjects list from an FSGD file.}
\usage{
read.md.subjects.from.fsgd(filepath)
}
\arguments{
\item{filepath}{character string, path to a FreeSurfer Group Descriptor (FSGD) file.}
}
\value{
vector of character strings, the subject identifiers
}
\description{
Read subjects list from an FSGD file.
}
\note{
This is not a parser for all data in an FSGD file.
}
\seealso{
\code{\link{demographics.to.fsgd.file}}
}
