% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/brainview_magick.R
\name{images.rescale.to.max.canvas}
\alias{images.rescale.to.max.canvas}
\title{Rescale all images canvas to match the largest one.}
\usage{
images.rescale.to.max.canvas(images, background = "white")
}
\arguments{
\item{images}{vector of magick images}

\item{background}{color string, like 'white' or '#00FF00'}
}
\value{
vector of magick images, the rescaled images
}
\description{
Rescale all images canvas to match the largest one.
}
\keyword{internal}
