% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis.R
\name{numverts.rh}
\alias{numverts.rh}
\title{Determine vertex count of right hemi from hemilist of surfaces or the count itself.}
\usage{
numverts.rh(surfaces)
}
\arguments{
\item{surfaces}{hemilist of surfaces, or a single integer, which will be interpreted as the number of vertices of the right hemisphere surface.}
}
\value{
integer, the number of vertices.
}
\description{
Determine vertex count of right hemi from hemilist of surfaces or the count itself.
}
