% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{scale01}
\alias{scale01}
\title{Scale given values to range 0..1.}
\usage{
scale01(x, ...)
}
\arguments{
\item{x}{the numeric data}

\item{...}{the numeric data}
}
\value{
the scaled data
}
\description{
Scale given values to range 0..1.
}
