% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spherical.R
\name{surf.radius.fsaverage}
\alias{surf.radius.fsaverage}
\title{Get pre-computed radius for fsaverage white surface.}
\usage{
surf.radius.fsaverage(maxr = TRUE)
}
\arguments{
\item{maxr}{logical, whether to return the maximum of the x, y, and z radii.}
}
\value{
If 'maxr' is TRUE, a numeric scalar representing the max of the x, y, and z radii, otherwise, a numerical vector of length 3 with all radii.
}
\description{
Get pre-computed radius for fsaverage white surface.
}
\note{
The coordinates are for the white surface and in surface space, i.e., based on the raw values stored in the \code{fsaverae/surf/lh.white} and \code{fsaverage/surf/rh.white} files, without applying any transformation.
}
\seealso{
\code{\link{surfs.props}}, which was used to compute the returned values.
}
\keyword{internal}
