% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{hasIn}
\alias{hasIn}
\title{Check for values in nested named lists}
\usage{
hasIn(named_list, listkeys)
}
\arguments{
\item{named_list}{a named list}

\item{listkeys}{vector of character strings, the nested names of the lists}
}
\value{
whether a non-NULL value exists at the path
}
\description{
Check for values in nested named lists
}
\examples{
   data = list("regions"=list("frontal"=list("thickness"=2.3, "area"=2345)));
   hasIn(data, c("regions", "nosuchregion"));   # FALSE

}
