% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis.R
\name{perform.na.mapping}
\alias{perform.na.mapping}
\title{Perform NA mapping for transparency}
\usage{
perform.na.mapping(data, map_to_NA)
}
\arguments{
\item{data}{numeric vector}

\item{map_to_NA}{the value or value range that should be mapped to \code{NA}. If a single value, only exactly this value is used. If two values, they are interpreted as a range, and a values between them are mapped to \code{NA}. If you pass \code{NULL}, the data are returned as-is.}
}
\value{
the mapped data
}
\description{
Usually this is done so that the \code{NA} values are plotted transparently, so your can see a background color through the respective colors.
}
\keyword{internal}
