% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coloredmesh.R
\name{is.fsbrain}
\alias{is.fsbrain}
\title{Check whether object is an fsbrain (S3)}
\usage{
is.fsbrain(x)
}
\arguments{
\item{x}{any \code{R} object}
}
\value{
TRUE if its argument is an fsbrain (that is, has "fsbrain" amongst its classes) and FALSE otherwise.
}
\description{
Check whether object is an fsbrain (S3)
}
