% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fsdir_abstraction_subject.R
\name{labeldata.from.mask}
\alias{labeldata.from.mask}
\title{Create labeldata from a mask.}
\usage{
labeldata.from.mask(mask, invert = FALSE)
}
\arguments{
\item{mask}{a logical vector}

\item{invert}{Whether to report the inverse the mask before determining the indices. Defaults to FALSE.}
}
\value{
labeldata. The list of indices which are TRUE in the mask (or the ones which FALSE if 'invert' is TRUE).
}
\description{
Create labeldata from a mask. This function is trivial and only calls \code{\link{which}} after performing basic sanity checks.
}
\seealso{
Other label data functions: 
\code{\link{group.label}()},
\code{\link{mask.from.labeldata.for.hemi}()},
\code{\link{subject.label}()}
}
\concept{label data functions}
