% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{vis.paths}
\alias{vis.paths}
\title{Visualize many paths.}
\usage{
vis.paths(coords_list, path_color = "#FF0000")
}
\arguments{
\item{coords_list}{list of \code{m} matrices, each \code{n} x 3 matrix must contain the 3D coords for one path.}

\item{path_color}{a color value, the color in which to plot the paths.}
}
\description{
Visualize many paths.
}
\note{
This function is a lot faster than calling \code{vis.path.along.verts} many times and having it draw each time.
}
