% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rglactions.R
\name{perform.rglactions}
\alias{perform.rglactions}
\title{Perform rglactions, like taking screenshots.}
\usage{
perform.rglactions(rglactions, at_index = NULL, silent = TRUE, ignore = c())
}
\arguments{
\item{rglactions, }{named list. A list in which the names are from a set of pre-defined actions. The values can be used to specify parameters for the action. See \code{\link{rglactions}}.}

\item{at_index}{integer, the index to use in case of vectorized entries. Allows using different output_images for different views or similar.}

\item{silent}{logical, whether to suppress messages}

\item{ignore}{vector of character strings, actions to ignore}
}
\description{
Take a list specifying actions, see \code{\link{rglactions}}, and execute them. This function should be called once an rgl scene has been setup and rendered. A typical use case is to save a screenshot of the scene.
}
\keyword{internal}
