% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_surface_background.R
\name{alphablend}
\alias{alphablend}
\title{Perform alpha blending for pairs of RGBA colors.}
\usage{
alphablend(front_color, back_color, silent = TRUE)
}
\arguments{
\item{front_color}{rgba color strings, the upper color layer or foreground}

\item{back_color}{rgba color strings, the lower color layer or background}

\item{silent}{logical, whether to suppress messages}
}
\value{
rgba color strings, the alpha-blended colors
}
\description{
Implements the \emph{over} alpha blending operation.
}
\references{
see the \emph{Alpha blending} section on https://en.wikipedia.org/wiki/Alpha_compositing
}
\seealso{
Other color functions: 
\code{\link{desaturate}()}
}
\concept{color functions}
