% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cbar.R
\name{plot.fsbrain.colorbar}
\alias{plot.fsbrain.colorbar}
\title{Draw a simple colorbar from colors.}
\usage{
\method{plot}{fsbrain.colorbar}(colors, horizontal = FALSE)
}
\arguments{
\item{colors}{vector of colors, no special ordering is assumed}

\item{horizontal}{logical, whether the colorbar should be plotted horizontally (or vertically).}
}
\description{
Draw a simple colorbar from colors.
}
\note{
This function assumes that there is an open plot, use \code{plot.new()} to create one before calling this function if that is not the case.
}
\keyword{internal}
