% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coloredmesh.R
\name{brain}
\alias{brain}
\title{Create fsbrain instance from 2 coloredmeshes.}
\usage{
brain(lh_cm, rh_cm)
}
\arguments{
\item{lh_cm}{coloredmesh for left hemisphere}

\item{rh_cm}{coloredmesh for right hemisphere}
}
\value{
fsbrain instance
}
\description{
Create fsbrain instance from 2 coloredmeshes.
}
\keyword{internal}
