% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fsdir_abstraction_group.R
\name{check.subjectslist}
\alias{check.subjectslist}
\title{Check whether the subjects_list looks good, warn if not.}
\usage{
check.subjectslist(
  subjects_list,
  subjects_dir = NULL,
  report_name = "subjects_list"
)
}
\arguments{
\item{subjects_list}{vector of character strings, the subject IDs.}

\item{subjects_dir}{optional, character string. The full path to the subjects directory containing all subjects. If given, extra checks will be performed, e.g., whether the subjects exist in this directory.}

\item{report_name}{character string, the variable name that should be used for the list in the messages.}
}
\description{
Check whether the subjects_list looks good, warn if not.
}
\note{
This function will stop if subjects dirs are missing.
}
\keyword{internal}
