% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{fs.home}
\alias{fs.home}
\title{Return FreeSurfer path.}
\usage{
fs.home()
}
\value{
the FreeSurfer path, typically what the environment variable \code{FREESURFER_HOME} points to.
}
\description{
Return FreeSurfer path.
}
\note{
This function will stop (i.e., raise an error) if the directory cannot be found.
}
