% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{gen.test.volume}
\alias{gen.test.volume}
\title{Generate test 3D volume of integers. The volume has an outer background area (intensity value 'bg') and an inner foreground areas (intensity value 200L).}
\usage{
gen.test.volume(vdim = c(256L, 256L, 256L), bg = NA)
}
\arguments{
\item{vdim}{integer vector of length 3, the dimensions}

\item{bg}{value to use for outer background voxels. Typically \code{0L} or \code{NA}.}
}
\value{
a 3d array of integers
}
\description{
Generate test 3D volume of integers. The volume has an outer background area (intensity value 'bg') and an inner foreground areas (intensity value 200L).
}
\note{
This function exists for software testing purposes only, you should not use it in client code.
}
