% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_multiview.R
\name{handle.rglactions.highlight.points}
\alias{handle.rglactions.highlight.points}
\title{Highlight requested points (if any), but apply given view rotation before doing so.}
\usage{
handle.rglactions.highlight.points(
  rglactions,
  angle_rad,
  x,
  y,
  z,
  hemi = "both"
)
}
\arguments{
\item{hemi}{character string, one of 'lh', 'rh' or 'both'. If lh or rh, plots only points from that hemi (if hemi info is available for the points).}
}
\description{
Highlight requested points (if any), but apply given view rotation before doing so.
}
\keyword{internal}
