% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geodesic.R
\name{geodesic.dists.to.vertex}
\alias{geodesic.dists.to.vertex}
\title{Simple internal wrapper around \code{Rvcg::vcgDijkstra} with function check.}
\usage{
geodesic.dists.to.vertex(mesh, v)
}
\arguments{
\item{mesh}{a tmesh3d instance.}

\item{v}{positive integer, a vertex index in the mesh.}
}
\value{
double vector with length equal to num vertices in the mesh, the geodesic distances from all other vertices to the query vertex \code{v}.
}
\description{
Simple internal wrapper around \code{Rvcg::vcgDijkstra} with function check.
}
