% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cbar.R
\name{cm.cbry}
\alias{cm.cbry}
\title{Get cyan blue red yellow colormap function.}
\usage{
cm.cbry()
}
\description{
Get cyan blue red yellow colormap function.
}
\note{
Returns a diverging palette with negative values in blue/cyan and positive ones in red/yellow, suitable for visualizing data that is centered around zero. Often used for clusters in neuroscience.
}
