% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_meshes.R
\name{is.Triangles3D}
\alias{is.Triangles3D}
\title{Check whether object is a Triangles3D instance}
\usage{
is.Triangles3D(x)
}
\arguments{
\item{x}{any \code{R} object}
}
\value{
TRUE if its argument is a Triangles3D instance (that is, has "Triangles3D" amongst its classes) and FALSE otherwise.
}
\description{
Check whether object is a Triangles3D instance
}
\keyword{internal}
