% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coloredmesh.R
\name{tmesh3d.to.fs.surface}
\alias{tmesh3d.to.fs.surface}
\title{Get an fs.surface brain mesh from an rgl tmesh3d instance.}
\usage{
tmesh3d.to.fs.surface(tmesh)
}
\arguments{
\item{tmesh}{a tmesh3d instance, see \code{rgl::tmesh3d} for details.}
}
\value{
an fs.surface instance, as returned by \code{subject.surface} or \code{freesurferformats::read.fs.surface}.
}
\description{
Get an fs.surface brain mesh from an rgl tmesh3d instance.
}
