% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_volume_3d.R
\name{apply.transform}
\alias{apply.transform}
\title{Apply matmult transformation to input.}
\usage{
apply.transform(object, matrix_fun)
}
\arguments{
\item{object}{numerical vector/matrix or Triangles3D instance, the coorindates or object to transform.}

\item{matrix_fun}{a 4x4 affine matrix or a function returning such a matrix. If \code{NULL}, the input is returned as-is. In many cases you way want to use a matrix computed from the header of a volume file, e.g., the \code{vox2ras} matrix of the respective volume. See the \verb{mghheader.*} functions in the \emph{freesurferformats} package to obtain these matrices.}
}
\value{
the input after application of the affine matrix (matrix multiplication)
}
\description{
Apply affine transformation, like a \emph{vox2ras_tkr} transformation, to input. This is just matrix multiplication for different input objects.
}
