% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/brain_igraph.R
\name{ensure.fs.surface}
\alias{ensure.fs.surface}
\title{Check whether parameter is an fs.surface instance.}
\usage{
ensure.fs.surface(surface)
}
\arguments{
\item{surface}{an fs.surface instance which will be returned as-is, a tmesh3d which will be converted to a surface using \code{\link[fsbrain]{tmesh3d.to.fs.surface}}, or a character string which will be interpreted as a file system path and loaded with \code{freesurferformats::read.fs.surface}. Anything else will stop with an error.}
}
\value{
an fs.surface instance, unless an error occurs.
}
\description{
Check whether parameter is an fs.surface instance.
}
\keyword{internal}
