% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_volume.R
\name{get.slice.indices}
\alias{get.slice.indices}
\title{Compute slice indices from slice definition.}
\usage{
get.slice.indices(voldim, axis, slices)
}
\arguments{
\item{voldim}{integer vector, the dimension of the volume}

\item{axis}{integer, the axis}

\item{slices}{slice index definition. If a vector of integers, interpreted as slice indices. If a single negative interger \code{-n}, interpreted as every \code{nth} slice, starting at slice 1. The character string 'all' or the value \code{NULL} will be interpreted as \emph{all slices}.}
}
\value{
integer vector, the computed slice indices. They are guaranteed to be valid indices into the volume.
}
\description{
Compute slice indices from slice definition.
}
\keyword{internal}
