% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cbar.R
\name{symmrange}
\alias{symmrange}
\title{Given data, compute symmetric range around zero.}
\usage{
symmrange(x)
}
\arguments{
\item{x}{the data, could be a range of course.}
}
\description{
Given data, compute symmetric range around zero.
}
\keyword{internal}
