% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{deg2rad}
\alias{deg2rad}
\title{Convert degree to radians}
\usage{
deg2rad(deg)
}
\description{
Convert degree to radians
}
\keyword{internal}
