% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_meshes.R
\name{get.rglstyle.parameters}
\alias{get.rglstyle.parameters}
\title{Produce the named list of style parameters from style definition.}
\usage{
get.rglstyle.parameters(renderable, style)
}
\arguments{
\item{renderable}{A renderable (or any list) which includes a 'style' key. If it does not include such a key, the 'default' style will be used.}

\item{style}{A style definition. Can be a character string like 'shiny' or 'from_mesh', or already a named list of material properties (which will be returned as-is).}
}
\value{
a style, resolved to a parameter list compatible with \code{\link[rgl]{material3d}}.
}
\description{
A style definition can be a character string like "shiny", already a parameter list, or a command like 'from_mesh' that tells us to get the style from the renderable. This function creates the final parameters from the definition and the renderable.
}
\keyword{internal}
