% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highlight.R
\name{highlight.points.spheres}
\alias{highlight.points.spheres}
\title{Draw small 3D spheres at given points.}
\usage{
highlight.points.spheres(coords, color = "#FF0000", radius = 1)
}
\arguments{
\item{coords}{double vector or nx3 double matrix, the xyz point coordinates.}

\item{color}{the sphere color, like \code{'#FF0000'} or \code{"red"}.}

\item{radius}{double, the sphere radius}
}
\description{
Draw small 3D spheres at given points.
}
\seealso{
Other 3d utility functions: 
\code{\link{highlight.vertices.spheres}()},
\code{\link{vertex.coords}()}
}
\concept{3d utility functions}
