% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coloredmesh.R
\name{print.fs.coloredmesh}
\alias{print.fs.coloredmesh}
\title{Print description of a brain coloredmesh (S3).}
\usage{
\method{print}{fs.coloredmesh}(x, ...)
}
\arguments{
\item{x}{brain surface with class \code{fs.coloredmesh}.}

\item{...}{further arguments passed to or from other methods}
}
\description{
Print description of a brain coloredmesh (S3).
}
