% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/smooth.R
\name{pervertexdata.smoothnn.compute.numiter}
\alias{pervertexdata.smoothnn.compute.numiter}
\title{Compute number of neighborhood smoothing iterations to reach requested fwhm.}
\usage{
pervertexdata.smoothnn.compute.numiter(surface, fwhm, is_template)
}
\arguments{
\item{surface}{an \code{fs.surface} instance}

\item{is_template}{logical, whether the surface belongs to a template subject}
}
\value{
integer, the iteration count
}
\description{
Compute number of neighborhood smoothing iterations to reach requested fwhm.
}
\note{
This function has been adapted from FreeSurfer and it is subject to the FreeSurfer software license.
}
\keyword{internal}
