% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spherical.R
\name{surfs.props}
\alias{surfs.props}
\title{Compute simple version of center and radius of 2 meshes.}
\usage{
surfs.props(lh, rh)
}
\arguments{
\item{lh}{\code{fs.surface} instance, the mesh for the left hemisphere. If a string, assumed to be the path to a surface mesh file that should be loaded.}

\item{rh}{\code{fs.surface} instance, the mesh for the right hemisphere. If a string, assumed to be the path to a surface mesh file that should be loaded.}
}
\value{
named list with entries 'center', 'radius', 'min' and 'max', each of which are numerical vectors of length 3, holding the x, y and z coordinates or values.
}
\description{
Compute simple version of center and radius of 2 meshes.
}
\note{
This function treats the 'lh' and 'rh' meshes as a single mesh, and computes the properties for this combined mesh.
}
\keyword{internal}
