% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{read.colorcsv}
\alias{read.colorcsv}
\title{Read colors from CSV file.}
\usage{
read.colorcsv(filepath)
}
\arguments{
\item{filepath}{character string, path to a CSV file containing colors}
}
\value{
vector of hex color strings
}
\description{
Read colors from CSV file.
}
