% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/optdata.R
\name{list_optional_data}
\alias{list_optional_data}
\title{Get file names available in package cache.}
\usage{
list_optional_data()
}
\value{
vector of strings. The file names available, relative to the package cache.
}
\description{
Get file names of optional data files which are available in the local package cache. You can access these files with get_optional_data_file().
}
