% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_meshes.R
\name{get.rglstyle.edges}
\alias{get.rglstyle.edges}
\title{Get the mesh edges visualization style parameters as a named list.}
\usage{
get.rglstyle.edges()
}
\value{
named list, style parameters that can be passed to \code{\link[rgl]{shade3d}} via \code{\link{do.call}}.
}
\description{
Mesh edges rendering style. Zoom in enough to see them.
}
\keyword{internal}
