% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fsdir_abstraction_group.R
\name{check.subjects.files}
\alias{check.subjects.files}
\title{Report subjects missing files}
\usage{
check.subjects.files(
  subjects_dir,
  subjects_list,
  sfiles = c("surf/lh.thickness", "surf/rh.thickness", "surf/lh.white", "surf/rh.white",
    "label/lh.aparc.annot", "label/rh.aparc.annot")
)
}
\arguments{
\item{subjects_dir}{string. The FreeSurfer SUBJECTS_DIR, i.e., a directory containing the data for all your subjects, each in a subdir named after the subject identifier.}

\item{subjects_list}{vector of strings. The subject identifiers.}

\item{sfiles}{vector of character strings, the file names, relative to the data dir of the subject.}
}
\value{
vector of character strings, the subjects missing any of the files.
}
\description{
Report subjects missing files
}
\keyword{internal}
