% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{fsbrain.set.default.figsize}
\alias{fsbrain.set.default.figsize}
\title{Set default figure size for fsbrain visualization functions.}
\usage{
fsbrain.set.default.figsize(width, height, xstart = 50L, ystart = 50L)
}
\arguments{
\item{width}{integer, default figure width in pixels}

\item{height}{integer, default figure height in pixels}

\item{xstart}{integer, default horizontal position of plot window on screen, left border is 0. The max value (right border) depends on your screen resolution.}

\item{ystart}{integer, default vertical position of plot window on screen, upper border is 0. The max value (lower border) depends on your screen resolution.}
}
\description{
Set default figure size for fsbrain visualization functions.
}
\note{
This function overwrites \code{options("fsbrain.rgloptions")}. Output size is limited by your screen resolution. To set your preferred figure size for future R sessions, you could call this function in your \code{'~/.Rprofile'} file.
}
