% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rglactions.R
\name{rglactions.has.key}
\alias{rglactions.has.key}
\title{Check for a key in names of rglactions.}
\usage{
rglactions.has.key(rglactions, key)
}
\arguments{
\item{rglactions, }{named list. A list in which the names are from a set of pre-defined actions, see \code{\link{rglactions}}. The values can be used to specify parameters for the action.}
}
\value{
logical, whether the \code{\link{rglactions}} instance has the requested key as a name.
}
\description{
Check for a key in names of rglactions.
}
\keyword{internal}
