% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/seg_stats.R
\name{sjld}
\alias{sjld}
\title{Get subjects list from subjects.txt file in dir.}
\usage{
sjld(subjects_dir)
}
\arguments{
\item{subjects_dir}{character string, existing subjects dir with a subjects.txt file containing one subject per line and no header line.}
}
\value{
named list with entries: 'd', the query subjects_dir (repeated from the parameter), 'l', vector of character strings, the subjects_list read from the file, 'f', the subjects_file.
}
\description{
Get subjects list from subjects.txt file in dir.
}
\note{
This function stops if the file does not exist or cannot be read.
}
