% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{demo}
\alias{demo}
\title{Show demo visualization to test whether fsbrain is setup correctly.}
\usage{
demo()
}
\description{
Show demo visualization to test whether fsbrain is setup correctly.
}
\note{
This function will try to download optional data from the internet (unless the data have already been downloaded).
}
\keyword{internal}
