% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/morph_atlas_agg.R
\name{mergehemi.annots}
\alias{mergehemi.annots}
\title{Merge the annotations from two hemispheres into one annot.}
\usage{
mergehemi.annots(lh_annot, rh_annot)
}
\arguments{
\item{lh_annot, }{annot. An annotation, as returned by freesurferformats::read.fs.annot().}

\item{rh_annot, }{annot. An annotation, as returned by freesurferformats::read.fs.annot().}
}
\value{
annot, the merged annotation.
}
\description{
Merge the annotations from two hemispheres into one annot.
}
\keyword{internal}
