% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/brainview_magick.R
\name{images.annotate}
\alias{images.annotate}
\title{Annotate image with text.}
\usage{
images.annotate(images, annotations, do_extend = TRUE, background = "white")
}
\arguments{
\item{images}{vector of magick images}

\item{annotations}{vector of character strings, the strings to print onto the tiles}

\item{do_extend}{logical, whether to add the space for the annotation text below the existing image tile}

\item{background}{color string, like 'white' or '#00FF00'}
}
\value{
vector of magick images, the annotated images
}
\description{
Annotate image with text.
}
\keyword{internal}
